@@ -6,7 +6,7 @@
 
   Author:  Illiya Yalovoy <yalovoy@gmail.com>
 
-  $Id: FWObjectDropArea.cpp,v 1.2 2006/09/15 05:48:21 vkurland Exp $
+  $Id: FWObjectDropArea.cpp,v 1.3 2006/10/22 04:39:36 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -39,6 +39,7 @@
 #include <qpainter.h>
 #include <qrect.h>
 #include <qpixmap.h>
+#include <qpixmapcache.h>
 #include <qrect.h>
 #include <qpopupmenu.h>
 
@@ -74,7 +75,14 @@ void FWObjectDropArea::paintEvent(QPaintEvent *ev)
     if (object!=NULL) 
     {
 
-        QPixmap pm = getPixmap(object);
+        QPixmap pm;
+        QString icn_file = 
+            Resources::global_res->getObjResourceStr(object, "icon-tree").c_str();
+        if ( ! QPixmapCache::find( icn_file, pm) )
+        {
+            pm = QPixmap::fromMimeSource( icn_file );
+            QPixmapCache::insert( icn_file, pm);
+        }
 
         tp.drawPixmap( ((w-pm.width())/2), (h/2)-(2+pm.height()) , pm );
         
@@ -123,20 +131,6 @@ void FWObjectDropArea::deleteObject()
     emit objectDeleted();
 }
 
-QPixmap FWObjectDropArea::getPixmap(FWObject *obj) const
-{
-    QPixmap pm;
-    string icn = "icon";
-    //if (pmattr == Neg)  icn="icon-neg";
-    //if (pmattr == Ref)  icn="icon-ref";
-    //if (pmattr == Tree) icn="icon-tree";
-    //
-    icn="icon-tree";
-    
-    return QPixmap::fromMimeSource( 
-        Resources::global_res->getObjResourceStr(obj, icn).c_str() );
-}
-
 void FWObjectDropArea::contextMenuEvent (QContextMenuEvent * e)
 {
     QPopupMenu *popup;
