@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWObjectPropertiesFactory.h,v 1.9 2006/02/19 07:49:48 vkurland Exp $
+  $Id: FWObjectPropertiesFactory.h,v 1.10 2007/02/04 03:58:59 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -47,11 +47,15 @@ class FWObjectPropertiesFactory {
     static QString getObjectPropertiesDetailed(libfwbuilder::FWObject *obj,
                                                bool showPath=false,
                                                bool tooltip=false,
-                                               bool accentName=true);
+                                               bool accentName=true,
+                                               bool richText=true);
     static QString getRuleActionProperties(libfwbuilder::PolicyRule *rule); 
     static QString getRuleActionPropertiesRich(libfwbuilder::PolicyRule *rule); 
     static QString getPolicyRuleOptions(libfwbuilder::Rule *rule);
     static QString getNATRuleOptions(libfwbuilder::Rule *rule);
+
+    static QString stripHTML(const QString &str);
+
 };
 
 #endif
