@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: FWWindow.cpp,v 1.212 2006/11/10 05:24:48 vkurland Exp $
+  $Id: FWWindow.cpp,v 1.215 2007/02/04 03:58:59 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -143,6 +143,7 @@
 #include <qwidgetstack.h>
 #include <qlistbox.h>
 #include <qeventloop.h>
+#include <qtextstream.h>
 
 using namespace libfwbuilder;
 using namespace std;
@@ -393,10 +394,9 @@ bool FWWindow::saveIfModified()
     return true;
 }
 
-QString FWWindow::chooseNewFileName(const QString &fname,
-                                    bool checkPresence,const QString &title)
+QString FWWindow::getDestDir(const QString &fname)
 {
-    QString destdir;
+    QString destdir = "";
 
     if (st->getWDir().isEmpty())
     {
@@ -414,12 +414,22 @@ QString FWWindow::chooseNewFileName(const QString &fname,
         } else
         {
             if (QFileInfo(fname).isDir()) destdir=fname;
-            else                      destdir = fname.left( fname.findRev('/',-1) );
+            else
+                destdir = fname.left( fname.findRev('/',-1) );
         }
     } else
     {
         destdir=st->getWDir();
     }
+    return destdir;
+}
+
+
+
+QString FWWindow::chooseNewFileName(const QString &fname,
+                                    bool checkPresence,const QString &title)
+{
+    QString destdir = getDestDir(fname);
 
     QString fn = QFileDialog::getSaveFileName( destdir,
        tr( "FWB Files (*.fwb);;All Files (*)" ),
@@ -1315,6 +1325,7 @@ void FWWindow::loadLibrary(const string &libfpath)
         FWObject *dobj = ndb->findInIndex(FWObjectDatabase::getDeletedObjectsId());
         if (dobj) ndb->remove(dobj, false);
 
+#if 0
         list<FWObject*> newLibs;
         newLibs= ndb->getByType(Library::TYPENAME);
 
@@ -1349,7 +1360,7 @@ void FWWindow::loadLibrary(const string &libfpath)
             for (list<FWObject*>::iterator i=duplicateLibs.begin(); i!=duplicateLibs.end(); i++)
                 ndb->remove(*i,false);
         }
-
+#endif
         MergeConflictRes mcr(this);
         db()->merge(ndb, &mcr);
 
@@ -1385,6 +1396,154 @@ void FWWindow::fileImport()
 //    addOnLibs->add( fname.latin1() );
 }
 
+
+void FWWindow::fileCompare()
+{
+    fd->reset();   // fd : find dialog
+
+    QMessageBox initial_question( "Firewall Builder",
+                    tr("This operation inspects two data files (either .fwb or .fwl) and finds conflicting objects. Conflicting objects have the same internal ID but different attributes. Two data files can not be merged, or one imported into another, if they contain such objects. This operation also helps identify changes made to objects in two copies of the same data file.<br><br>This operation does not find objects present in one file but not in the other, such objects present no problem for merge or import operations.<br><br>This operation works with two external files, neither of which needs to be opened in the program. Currently opened data file is not affected by this operation and objects in the tree do not change.<br><br>Do you want to proceed ?"),
+                    QMessageBox::Information,
+                    QMessageBox::Yes | QMessageBox::Default,
+                    QMessageBox::No,
+                    QMessageBox::Escape );
+
+    initial_question.setTextFormat( Qt::RichText );
+    if (initial_question.exec() != QMessageBox::Yes) return;
+
+
+    QString fname1 = QFileDialog::getOpenFileName( st->getWDir(), 
+       "Firewall Builder 2 files (*.fwb);;FWB Library Files (*.fwl);;All Files (*)",
+                          this, 0,
+                          tr("Choose the first file") );
+
+    if (fname1.isEmpty()) return;   // Cancel
+
+    QString fname2 = QFileDialog::getOpenFileName( st->getWDir(), 
+       "Firewall Builder 2 files (*.fwb);;FWB Library Files (*.fwl);;All Files (*)",
+                          this, 0,
+                          tr("Choose the second file") );
+
+    if (fname2.isEmpty()) return;   // Cancel
+
+    MessageBoxUpgradePredicate upgrade_predicate;
+
+    FWObjectDatabase *db1;
+    FWObjectDatabase *db2;
+    FWObject *dobj;
+
+    try
+    {
+        db1 = new FWObjectDatabase();
+        db1->load(fname1,  &upgrade_predicate,  librespath);
+
+        dobj = db1->findInIndex(FWObjectDatabase::getDeletedObjectsId());
+        if (dobj) db1->remove(dobj, false);
+    } catch(FWException &ex)
+    {
+        QMessageBox::warning(
+            this,"Firewall Builder", 
+            tr("Error loading file %1:\n%2").
+                 arg(fname1).arg(ex.toString().c_str()),
+            tr("&Continue"), QString::null,QString::null,
+            0, 1 );
+        return;
+    }
+
+    try
+    {
+        db2 = new FWObjectDatabase();
+        db2->load(fname2,  &upgrade_predicate,  librespath);
+
+        dobj = db2->findInIndex(FWObjectDatabase::getDeletedObjectsId());
+        if (dobj) db2->remove(dobj, false);
+    } catch(FWException &ex)
+    {
+        QMessageBox::warning(
+            this,"Firewall Builder", 
+            tr("Error loading file %1:\n%2").
+                 arg(fname2).arg(ex.toString().c_str()),
+            tr("&Continue"), QString::null,QString::null,
+            0, 1 );
+        return;
+    }
+
+    try
+    {
+        // CompareObjectsDialog is just like ObjConflictResolutionDialog
+        // except it always returns 'accepted' and keeps record
+        // of all object differences so we can print report in the end
+
+        CompareObjectsDialog cod(this);
+        db1->merge(db2, &cod);
+        list<QString> report = cod.getReport();
+
+        delete db1;
+        delete db2;
+
+        ostringstream str;
+        str << cod.getNumberOfConflicts();
+
+        QMessageBox mb( "Firewall Builder",
+                        tr("Total number of conflicting objects: %1.\nDo you want to generate report?").arg(str.str()),
+                        QMessageBox::Information,
+                        QMessageBox::Yes | QMessageBox::Default,
+                        QMessageBox::No,
+                        QMessageBox::Escape );
+
+        if (mb.exec()== QMessageBox::Yes)
+        {
+            // save report to a file
+
+            QString destdir = getDestDir(fname1);
+
+            QString fn = QFileDialog::getSaveFileName( destdir,
+                           tr( "TXT Files (*.txt);;All Files (*)" ),
+                           this, 0,
+                           tr("Choose name and location for the report file"));
+
+            if (fwbdebug)
+                qDebug( QString("Saving report to %1").arg(fn) );
+
+            if (fn.isEmpty() ) return ;   // Cancel
+
+            if (fn.findRev(".txt",-1,false)==-1)
+            {
+                fn+=".txt";
+            }
+
+            QFile report_file(fn);
+            if (report_file.open(IO_WriteOnly))
+            {
+                QTextStream report_stream(&report_file);
+                for (list<QString>::iterator i=report.begin(); i!=report.end(); ++i)
+                {
+                    report_stream << *i;
+                }
+                report_file.close();
+            } else 
+            {
+                QMessageBox::critical(
+                    this,"Firewall Builder", 
+                    tr("Can not open report file for writing. File '%1'").arg(fn),
+                    tr("&Continue"), QString::null,QString::null,
+                    0, 1 );
+            }
+            
+        }
+
+    } catch(FWException &ex)
+    {
+        QMessageBox::warning(
+            this,"Firewall Builder", 
+            tr("Unexpected error comparing files %1 and %2:\n%3").
+                 arg(fname1).arg(fname2).arg(ex.toString().c_str()),
+            tr("&Continue"), QString::null,QString::null,
+            0, 1 );
+    }
+
+}
+
 void FWWindow::findExternalRefs(FWObject *lib,
                                        FWObject *root,
                                        list<FWReference*> &extRefs)
@@ -1820,10 +1979,10 @@ void FWWindow::deleteFirewall(FWObject *fw)
     if (visibleFirewall==fw)  visibleFirewall=NULL;
 }
 
-void FWWindow::setPolicyBranchTabName(PolicyRule *rule)
+void FWWindow::setPolicyBranchTabName(RuleSet *subset)
 {
-    RuleSet *subset = rule->getBranch();
-    assert(subset);
+    assert(subset!=NULL);
+    PolicyRule *rule = PolicyRule::cast(subset->getParent());
     RuleSetView *rsv = ruleSetViews[subset];
     assert(rsv);
     FWOptions *ropt = rule->getOptionsObject();
@@ -1831,8 +1990,11 @@ void FWWindow::setPolicyBranchTabName(PolicyRule *rule)
     ruleSets->changeTab(rsv, tr("Policy/%1").arg(branchName) );
 }
 
-void FWWindow::addPolicyBranchTab(PolicyRule *rule)
+void FWWindow::addPolicyBranchTab(RuleSet *subset)
 {
+    assert(subset!=NULL);
+
+    PolicyRule *rule = PolicyRule::cast(subset->getParent());
     FWOptions *ropt = rule->getOptionsObject();
     QString branchName = ropt->getStr("branch_name").c_str();
 
@@ -1841,33 +2003,30 @@ void FWWindow::addPolicyBranchTab(PolicyRule *rule)
     QApplication::eventLoop()->processEvents(QEventLoop::ExcludeUserInput,1000);
     if (fwbdebug)  qDebug("FWWindow::reopenFirewall()   adding branch tab");
 
-    Policy *subset = Policy::cast(rule->getBranch());
-    if (subset==NULL)
-    {
-        subset = new Policy();
-        rule->add(subset);
-    }
+//    if (subset==NULL)
+//    {
+//        subset = new Policy();
+//        rule->add(subset);
+//    }
     if (ruleSetViews.count(subset)==0)
     {
-        RuleSetView *rsv;
-        ruleSets->addTab(rsv=new PolicyView(subset,NULL),
-                         "Branch"); // temporary name
+        RuleSetView *rsv = new PolicyView(Policy::cast(subset),NULL);
+        ruleSets->addTab(rsv,"Branch"); // temporary name
         ruleSetViews[subset]=rsv;
     }
-    setPolicyBranchTabName(rule);
+    setPolicyBranchTabName(subset);
 
     for (FWObject::iterator i=subset->begin(); i!=subset->end(); i++)
     {
         PolicyRule *srule = PolicyRule::cast(*i);
         if (srule->getAction() == PolicyRule::Branch)
-            addPolicyBranchTab(srule);
+            addPolicyBranchTab(srule->getBranch());
     }
 
 }
 
-void FWWindow::removePolicyBranchTab(PolicyRule *rule)
+void FWWindow::removePolicyBranchTab(RuleSet *subset)
 {
-    RuleSet *subset = rule->getBranch();
     if (subset==NULL) return;
     RuleSetView *rsv = ruleSetViews[subset];
     assert(rsv);
@@ -1877,8 +2036,11 @@ void FWWindow::removePolicyBranchTab(PolicyRule *rule)
 
 void FWWindow::scheduleRuleSetRedraw()
 {
-    ruleSetRedrawPending = true;
-    QTimer::singleShot( 0, this, SLOT(redrawRuleSets()) );
+    if (!ruleSetRedrawPending)
+    {
+        ruleSetRedrawPending = true;
+        QTimer::singleShot( 0, this, SLOT(redrawRuleSets()) );
+    }
 }
 
 void FWWindow::redrawRuleSets()
@@ -1927,7 +2089,7 @@ void FWWindow::reopenFirewall()
     {
         PolicyRule *rule = PolicyRule::cast(*i);
         if (rule->getAction() == PolicyRule::Branch)
-            addPolicyBranchTab(rule);
+            addPolicyBranchTab(rule->getBranch());
     }
 
 // let the GUI process events to display new tab(s)
@@ -2424,7 +2586,7 @@ void FWWindow::openEditorPanel()
 void FWWindow::ruleSetTabChanged(QWidget* w)
 {
     if (fwbdebug)
-        qDebug("FWWindow::ruleSetTabChanged:  w=%x ruleSetTabIndex=%d changingTabs=%d",
+        qDebug("FWWindow::ruleSetTabChanged:  w=%p ruleSetTabIndex=%d changingTabs=%d",
                w,ruleSetTabIndex,changingTabs);
 
     if (changingTabs) return;
