@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectTreeView.cpp,v 1.37 2006/10/22 18:20:04 vkurland Exp $
+  $Id: ObjectTreeView.cpp,v 1.39 2007/01/08 03:24:48 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -76,6 +76,24 @@ ObjectTreeView::ObjectTreeView(QWidget* parent, const char * name, WFlags f) :
     expandOrCollapse = false;
     Lockable = false;
     Unlockable = false;
+    visible = false;
+    /*
+     * note about process_mouse_release_event
+     *
+     * we use mouseReleaseEvent event to switch object opened in the
+     * editor panel (i.e. we open new object when mouse button is
+     * released rather than when it is pressed). This allows us to
+     * start drag without switching object in the editor. The problem
+     * is that mouseReleaseEvent is received in this widget after the
+     * d&d ends with a drop somewhere else, which triggers call to
+     * contentsMouseReleaseEvent which switches object in the
+     * editor. This is undesired when the editor shows a group and we
+     * try to drag and drop an object into that group. Flag
+     * process_mouse_release_event is used to suppress object
+     * switching when contentsMouseReleaseEvent is called after
+     * successfull drop.
+     */
+    process_mouse_release_event = true;
 
     connect( this, SIGNAL(currentChanged(QListViewItem*)),
              this, SLOT(currentChanged(QListViewItem*)) );
@@ -251,13 +269,12 @@ void ObjectTreeView::updateTreeItems()
  */
 QDragObject* ObjectTreeView::dragObject()
 {
-    if (fwbdebug)
-        qDebug("ObjectTreeView::dragObject");
+    if (fwbdebug) qDebug("ObjectTreeView::dragObject");
 
     QListViewItem *ovi = currentItem();
     ObjectTreeViewItem *otvi=dynamic_cast<ObjectTreeViewItem*>(ovi);
 
-    FWObject *obj = getCurrentObject();
+    FWObject *current_obj = getCurrentObject();
 
 /* can't drag system folders 
 
@@ -271,7 +288,7 @@ QDragObject* ObjectTreeView::dragObject()
     if (FWBTree::isSystem(obj)) return NULL;
 */
     QString icn =
-        Resources::global_res->getObjResourceStr(obj, "icon-ref").c_str();
+        Resources::global_res->getObjResourceStr(current_obj, "icon-ref").c_str();
 
     vector<FWObject*> so = getSimplifiedSelection();
 
@@ -315,9 +332,46 @@ QDragObject* ObjectTreeView::dragObject()
                          QPoint( pm.rect().width() / 2,
                                  pm.rect().height() / 2 ) );
 
-// lets try to always reset selection
-    setSelected(otvi,false);
-    resetSelection();
+/*
+ * This fragment returns selection in the tree back to the object that
+ * was selected before drag operation has started. This help in the
+ * following case:
+ *
+ *  - open a group for editing (group is selected in the tree)
+ *  - left-click on another object in the tree, start dragging it
+ *
+ * at this point selection in the tree returns to the group, so when
+ * user finishes d&d operation, the selection in the tree is consisten
+ * with object currently opened in the editor panel.
+ *
+ * There is a problem with this however. If user wants to put an
+ * object from a different library into the group, they have to switch
+ * to that library before doing d&d. When they switch, ObjectTree
+ * shown in the left panel becomes different from the tree in which
+ * the group is located. When d&d finishes, the ObjectTree object
+ * receives contentsMouseReleaseEvent event. Since it is not the right
+ * tree object, it can not properly restore selection and choses an
+ * object that was previously opened in that tree, which in turn
+ * changes the object opened in the editor panel. To make things
+ * worse, this event is only delivered to the tree object on Mac OS X.
+ *
+ * 
+ */
+    if (fwbdebug) qDebug("ObjectTreeView::dragObject()  this=%p visible=%d",
+                         this,visible);
+
+    FWObject *edit_obj = oe->getOpened();
+
+    if (oe->isVisible() && 
+        dragobj.size()==1 &&
+        edit_obj!=NULL &&
+        current_obj->getLibrary()==edit_obj->getLibrary() )
+    {
+        if (fwbdebug) qDebug("ObjectTreeView::dragObject() reset selection");
+        setSelected(otvi,false);
+        resetSelection();
+    }
+
 #if 0
     /*
      * need to reset selection if:
@@ -488,6 +542,7 @@ void ObjectTreeView::contentsMousePressEvent( QMouseEvent *e )
         qDebug("ObjectTreeView::contentsMousePressEvent");
 
     second_click = false;
+    process_mouse_release_event = true;
 
     if (fwbdebug)
     {
@@ -516,14 +571,38 @@ void ObjectTreeView::contentsMousePressEvent( QMouseEvent *e )
  * on a single click.
  *
  * uncomment the line that starts timer for mode #1.
+ *
+ *
+ * we use mouseReleaseEvent event to switch object opened in the
+ * editor panel (i.e. we open new object when mouse button is released
+ * rather than when it is pressed). This allows us to start drag
+ * without switching object in the editor. The problem is that
+ * mouseReleaseEvent is received in this widget after the d&d ends
+ * with a drop somewhere else, which triggers call to
+ * contentsMouseReleaseEvent which switches object in the editor. This
+ * is undesired when the editor shows a group and we try to drag and
+ * drop an object into that group. Flag process_mouse_release_event is
+ * used to suppress object switching when contentsMouseReleaseEvent is
+ * called after successfull drop.
+ *
  */
 void ObjectTreeView::contentsMouseReleaseEvent( QMouseEvent *e )
 {
     if (fwbdebug)
-        qDebug("ObjectTreeView::contentsMouseReleaseEvent 1");
+        qDebug("ObjectTreeView::contentsMouseReleaseEvent 1 this=%p  process_mouse_release_event=%d",
+               this,process_mouse_release_event);
 
     QListView::contentsMouseReleaseEvent(e);
 
+
+    if (!process_mouse_release_event)
+    {
+        // just do not switch object in the editor, otherwise
+        // process this event as usual
+        process_mouse_release_event = true;
+        return;
+    }
+
     if (fwbdebug)
         qDebug("ObjectTreeView::contentsMouseReleaseEvent 2 selectedObjects.size()=%d getCurrentObject()=%p current object %s",
                selectedObjects.size(),
