@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: instDialog.cpp,v 1.95 2006/10/21 06:19:36 vkurland Exp $
+  $Id: instDialog.cpp,v 1.99 2007/01/30 16:48:10 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -49,6 +49,7 @@
 #include <qapplication.h>
 #include <qeventloop.h>
 #include <qfile.h>
+#include <qdir.h>
 #include <qmessagebox.h>
 #include <qspinbox.h>
 #include <qgroupbox.h>
@@ -224,74 +225,22 @@ void instDialog::prepareInstallerOptions(bool fPix)
 
     QString platform=cnf.fwobj->getStr("platform").c_str();
 
-    cnf.rollbackTimeUnit=
-        Resources::getTargetOptionStr(cnf.fwobj->getStr("host_OS"),
-                                      "activation/timeout_units").c_str();
-    if ((platform!="pix" && platform!="fwsm" && !batchInstall->isChecked()) ||
-        (!fPix && batchInstall->isChecked()) )
-    {
-        dlg->incr->hide();
-        dlg->test->hide();
-        dlg->saveDiff->hide();
-        dlg->saveStandby->hide();
-        dlg->backupConfigFile->hide();
-        dlg->backupConfigFileLbl->hide();
-        dlg->epwd->hide();
-        dlg->epwdLbl->hide();
-
-        dlg->PIXgroupBox->hide();
-    } else
-    {
-// if PIX or FWSM:
-        dlg->copyFWB->hide();
-
-        dlg->incr->show();
-        dlg->test->show();
-        dlg->saveDiff->show();
-        dlg->saveStandby->show();
-        dlg->backupConfigFile->show();
-        dlg->backupConfigFileLbl->show();
-        dlg->epwd->show();
-        dlg->epwdLbl->show();
-
-        dlg->PIXgroupBox->show();
-    }
-
-/* hide anyway, diff does not work for pix 6.3(3) */
-    dlg->saveDiff->hide();
-
-    //progressBar->hide();
-    dlg->stripComments->hide();
-    dlg->compressScript->hide();
-
-   //if (platform=="pix" || platform=="fwsm")
-   //{
-   //    progressBar->show();
-   //    stripComments->show();
-   //}
-
-    if (cnf.fwobj->getStr("host_OS")=="linksys")
-    {
-        dlg->compressScript->show();
-    }
-
-
-   dlg->uname->setFocus();
-   dlg->uname->setText( cnf.user );
-   dlg->incr->setChecked( cnf.incremental );
-   dlg->test->setChecked( cnf.dry_run );
-   dlg->backupConfigFile->setText( cnf.backup_file );
-   dlg->saveDiff->setChecked( cnf.save_diff );
-   dlg->saveStandby->setChecked( cnf.saveStandby );
-   dlg->altAddress->setText( cnf.maddr );
-   dlg->quiet->setChecked( cnf.quiet );
-   dlg->verbose->setChecked( cnf.verbose );
-   dlg->stripComments->setChecked( cnf.stripComments );
-   dlg->compressScript->setChecked( cnf.compressScript );
-   dlg->copyFWB->setChecked( cnf.copyFWB );
-   dlg->testRun->setChecked( cnf.testRun );
-   dlg->rollback->setChecked( cnf.rollback );
-   dlg->rollbackTime->setValue( cnf.rollbackTime );
+    dlg->uname->setFocus();
+    dlg->uname->setText( cnf.user );
+    dlg->incr->setChecked( cnf.incremental );
+    dlg->test->setChecked( cnf.dry_run );
+    dlg->backupConfigFile->setText( cnf.backup_file );
+    dlg->saveDiff->setChecked( cnf.save_diff );
+    dlg->saveStandby->setChecked( cnf.saveStandby );
+    dlg->altAddress->setText( cnf.maddr );
+    dlg->quiet->setChecked( cnf.quiet );
+    dlg->verbose->setChecked( cnf.verbose );
+    dlg->stripComments->setChecked( cnf.stripComments );
+    dlg->compressScript->setChecked( cnf.compressScript );
+    dlg->copyFWB->setChecked( cnf.copyFWB );
+    dlg->testRun->setChecked( cnf.testRun );
+    dlg->rollback->setChecked( cnf.rollback );
+    dlg->rollbackTime->setValue( cnf.rollbackTime );
 
     bool f = dlg->testRun->isChecked();
     if (cnf.fwobj->getStr("host_OS")=="pix_os" || cnf.fwobj->getStr("host_OS")=="fwsm_os")
@@ -304,6 +253,40 @@ void instDialog::prepareInstallerOptions(bool fPix)
     dlg->rollbackTime->setEnabled(f);
     dlg->rollbackTimeUnit->setEnabled(f);
     
+    QSize pix_options_frame_size = dlg->PIXgroupBox->sizeHint();
+
+    cnf.rollbackTimeUnit=
+        Resources::getTargetOptionStr(cnf.fwobj->getStr("host_OS"),
+                                      "activation/timeout_units").c_str();
+    if ((platform!="pix" && platform!="fwsm" && !batchInstall->isChecked()) ||
+        (!fPix && batchInstall->isChecked()) )
+    {
+        dlg->hidePIXOptions();
+    }
+
+/* hide anyway, diff does not work for pix 6.3(3) */
+    //dlg->hideOption( dlg->saveDiff );
+    dlg->saveDiff->hide();
+
+    //progressBar->hide();
+    //dlg->hideOption( dlg->stripComments );
+    dlg->stripComments->hide(); 
+    //dlg->compressScript->hide();
+
+    //if (platform=="pix" || platform=="fwsm")
+    //{
+    //    progressBar->show();
+    //    stripComments->show();
+    //}
+
+    if (cnf.fwobj->getStr("host_OS")!="linksys")
+    {
+        dlg->compressScript->hide();
+        //dlg->hideOption( dlg->compressScript );
+    }
+
+    //dlg->adjustSize();
+    //dlg->setFixedHeight( dlg->minimumSizeHint().height() );
 
 /* we initialize these in FWBSettings constructor on Unix, but do not
  * do it on Windows since there is no standard ssh package there. User
@@ -639,6 +622,11 @@ void instDialog::selected(const QString &title)
     lastPage=indexOf( currentPage() );
 }
 
+QString instDialog::getFullPath(instConf &cnf, const QString &file )
+{
+    if (QDir::isRelativePath(file)) return cnf.wdir + "/" + file;
+    else return file;
+}
 
 bool instDialog::doInstallPage()
 {
@@ -724,7 +712,8 @@ bool instDialog::doInstallPage()
         }
 /* read manifest from the conf file */
 
-        QFile cf(  cnf.wdir+"/"+cnf.conffile );
+        QString conffile_path = getFullPath(cnf,cnf.conffile);
+        QFile cf( conffile_path  );
         if (cf.open( IO_ReadOnly ) )
         {
             QTextStream stream(&cf);
@@ -753,7 +742,7 @@ bool instDialog::doInstallPage()
         } else
         {
             QMessageBox::critical(this, "Firewall Builder",
-                 tr("File %1 not found.").arg(cnf.wdir+"/"+cnf.conffile),
+                 tr("File %1 not found.").arg(conffile_path),
                                   tr("&Continue") );
             return false;
         }
@@ -904,8 +893,20 @@ QString instDialog::getActivationCmd()
 /* replace macros in activation command:
  *
  * %FWSCRIPT%, %FWDIR%, %FWBPROMPT%, %RBTIMEOUT%
+ *
+ * check if cnf.conffile is a full path. If it is, strip the path part
+ * and use only the file name for %FWSCRIPT%
  */
-    cmd.replace("%FWSCRIPT%",cnf.conffile);
+    QString clean_conffile = cnf.conffile.section(QDir::separator(),-1);
+    if (fwbdebug)
+    {
+        qDebug("Macro substitutions:");
+        qDebug(QString("  cnf.conffile=%1").arg(cnf.conffile));
+        qDebug(QString("  %%FWSCRIPT%%=%1").arg(clean_conffile));
+        qDebug(QString("  %%FWDIR%%=%1").arg(cnf.fwdir));
+    }
+
+    cmd.replace("%FWSCRIPT%",clean_conffile);
     cmd.replace("%FWDIR%",cnf.fwdir);
     cmd.replace("%FWBPROMPT%",fwb_prompt);
 
@@ -934,16 +935,16 @@ void instDialog::initiateCopy(const QString &file)
     
     std::ifstream   *wfile;
 
-    QString ff = cnf.wdir+"/"+file;
+    QString file_with_path = getFullPath(cnf,file);
 
-    wfile = new ifstream(ff.latin1());
+    wfile = new ifstream(file_with_path.latin1());
     if ( ! *wfile)
     {
-        ff = file;  // .fwb file path already includes wdir
-        wfile = new ifstream(ff.latin1());
+        file_with_path = file;  // .fwb file path already includes wdir
+        wfile = new ifstream(file_with_path.latin1());
         if ( ! *wfile)
         {
-            addToLog(QObject::tr("Can not open file %1").arg(ff));
+            addToLog(QObject::tr("Can not open file %1").arg(file_with_path));
             delete wfile;
             return;
         }
@@ -1018,16 +1019,28 @@ void instDialog::initiateCopy(const QString &file)
 /* replace macros in activation command:
  *
  * %FWSCRIPT%, %FWDIR%, %FWBPROMPT%, %RBTIMEOUT%
+ *
+ * check if cnf.conffile is a full path. If it is, strip the path part
+ * and use only the file name for %FWSCRIPT%
  */
+    QString clean_conffile = cnf.conffile.section(QDir::separator(),-1);
+    if (fwbdebug)
+    {
+        qDebug("Macro substitutions:");
+        qDebug(QString("  cnf.conffile=%1").arg(cnf.conffile));
+        qDebug(QString("  %%FWSCRIPT%%=%1").arg(clean_conffile));
+        qDebug(QString("  %%FWDIR%%=%1").arg(cnf.fwdir));
+    }
+
     cmd.replace("\n","");
-    cmd.replace("%FWSCRIPT%",file);
+    cmd.replace("%FWSCRIPT%",clean_conffile);
     cmd.replace("%FWDIR%",cnf.fwdir);
     cmd.replace("%FWBPROMPT%",fwb_prompt);
 
     args.push_back(cmd);
         
     addToLog( tr("\nCopying %1 -> %2:%3\n")
-                .arg(cnf.wdir+"/"+file).arg(cnf.maddr).arg(cnf.fwdir) );
+                .arg(file_with_path).arg(cnf.maddr).arg(cnf.fwdir) );
 
     if (cnf.verbose) displayCommand(args);
 
@@ -1393,7 +1406,11 @@ bool instDialog::testFirewall(Firewall *fw)
     if (ofname.isEmpty()) ofname = QString(fw->getName().c_str()) + ".fw";
 
     QString fwfname = getFileDir( mw->getRCS()->getFileName() ) + "/" + ofname;
-    if ( !QFile::exists(fwfname) )
+
+/* bug #1617501: "Install fails after compile". Check ofname, just in
+ * case user put full path name for the output script name in options
+ */
+    if ( !QFile::exists(fwfname) && !QFile::exists(ofname))
     {
 /* need to recompile */
         addToLog(tr("Firewall isn't compiled."));
@@ -1401,6 +1418,7 @@ bool instDialog::testFirewall(Firewall *fw)
         return false;
     }
     
+
     args.clear();
     
    
