@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler_ipt.h,v 1.25 2006/11/10 05:24:48 vkurland Exp $
+  $Id: PolicyCompiler_ipt.h,v 1.27 2007/01/05 06:47:06 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -300,6 +300,12 @@ namespace fwcompiler {
 	 */
         DECLARE_POLICY_RULE_PROCESSOR(setChainIfTagInPrerouting);
 
+	/**
+	 * set chain for mangle table
+	 */
+        DECLARE_POLICY_RULE_PROCESSOR(setChainForMangle);
+
+        
 	/**
 	 * split rule if action is Tag and connmark option is activated
 	 */
@@ -557,6 +563,13 @@ namespace fwcompiler {
          */
         DECLARE_POLICY_RULE_PROCESSOR(decideOnChainForClassify);
 
+        /**
+         * find non-terminating targets (such as MARK and
+         * CLASSIFY). Put such rule in a separate chain and pass
+         * control to it using "-g"
+         */
+        DECLARE_POLICY_RULE_PROCESSOR(splitNonTerminatingTargets);
+
 	/**
 	 * decides what chain this rule should go to if it has not
 	 * been decided in decideOnChainIfFW
